﻿'   BKConsole 1.0
'     Copyright (c) 2012 - Vanderhaegen Bart
'
'
'   BKConsole is a very small console application to demonstrate the possibilities of the BKcomm class,
'   a class to control the programmable power supply BK1786B from BK Instruments.
'
'   This program is still in an experimental phase!
'
'
'

Imports System.Threading

Module modMain

    ' The BK instrument
    Private myBK As New BKcomm()

    ' Class variables
    Private port As Integer
    Private baud As Integer
    Private addr As Integer


    ''' <summary>
    ''' loadSettings() loads the settings from the data file
    ''' </summary>
    ''' <remarks></remarks>
    Sub loadSettings()
        port = My.Settings.port
        baud = My.Settings.baud
        addr = My.Settings.addr
    End Sub



    ''' <summary>
    ''' saveSettings() writes the settings to the data file
    ''' </summary>
    ''' <remarks></remarks>
    Sub saveSettings()
        My.Settings.port = port
        My.Settings.baud = baud
        My.Settings.addr = addr
        My.Settings.Save()
    End Sub



    ''' <summary>
    ''' askInput() asks the user for a decimal number input. It is used for the setup menu.
    ''' </summary>
    ''' <param name="text">the message to show</param>
    ''' <param name="minValue">minimum allowed value</param>
    ''' <param name="maxValue">maximum allowed value</param>
    ''' <returns>the value the user entered</returns>
    ''' <remarks></remarks>
    Function askInput(ByVal text As String, ByVal minValue As Integer, ByVal maxValue As Integer) As Integer

        Dim validValue As Boolean = False
        Dim myInteger As Integer
        Do
            Dim newCmd As String
            newCmd = getInput(text & " > ")

            If Integer.TryParse(newCmd, myInteger) Then
                If myInteger >= minValue And myInteger <= maxValue Then
                    validValue = True
                Else
                    Console.WriteLine("value outside allowed range (" & minValue & "-" & maxValue & ")")
                End If
            Else
                Console.WriteLine("value is no valid integer number")
            End If

        Loop While validValue = False

        Return myInteger

    End Function



    ''' <summary>
    ''' Main() is the main sub of our program
    ''' </summary>
    ''' <remarks></remarks>
    Sub Main()

        ' Initial text
        Console.WriteLine("BK1786B Console Application")
        Console.WriteLine("  Copyright (c) 2012 - Vanderhaegen Bart")
        Console.WriteLine("")

        ' Load settings (by default all settings are -1)
        loadSettings()

        ' Endless loop
        Do
            ' Asks input
            Dim newCmd As String
            newCmd = getInput("BK > ")

            ' Validate the input
            Select Case newCmd
                Case "CONNECT"

                    If port = -1 Or baud = -1 Or addr = -1 Then
                        Console.WriteLine("please run setup first! type SETUP")
                    Else
                        If myBK.connect(port, baud, addr) Then
                            Console.WriteLine("Connected to the BK instrument")
                            Console.WriteLine("Make sure the instrument is in remote mode! type REMOTE.")
                        Else
                            Console.WriteLine("Connection Failed. Call SETUP to view parameters")
                        End If
                    End If

                Case "SETUP"
                    port = askInput("PORT", 1, 32)
                    baud = askInput("BAUD", 4800, 38400)
                    addr = askInput("ADDR", 0, 255)
                    saveSettings()

                Case "DISCONNECT"
                    myBK.disconnect()
                    Console.WriteLine("Disconnected...")

                Case "CLS"
                    Console.Clear()

                Case "REMOTE"
                    If myBK.setRemoteControlMode(BKcomm.remoteControlMode.REMOTE_OPERATION) Then
                        Console.WriteLine("Device switched to remote mode.")
                    Else
                        Console.WriteLine("Error: command could not be sent. Are you connected?")
                    End If

                Case "EXIT"
                    End

                Case "LOCAL"
                    If myBK.setRemoteControlMode(BKcomm.remoteControlMode.FRONT_PANEL_OPERATION) Then
                        Console.WriteLine("Device switched to front panel mode.")
                    Else
                        Console.WriteLine("Error: command could not be sent. Are you connected?")
                    End If

                Case "ON"
                    If myBK.enableOutput(True) Then
                        Console.WriteLine("Device output is enabled")
                    Else
                        Console.WriteLine("Error: command could not be sent. Are you connected?")
                    End If

                Case "OFF"
                    If myBK.enableOutput(False) Then
                        Console.WriteLine("Device output is disabled")
                    Else
                        Console.WriteLine("Error: command could not be sent. Are you connected?")
                    End If

                Case "P"
                    Dim volt As Double
                    Dim amps As Double

                    BKcomm.mV_to_V(volt)
                    volt = myBK.getPresentVoltage()
                    Thread.Sleep(100)
                    amps = myBK.getPresentCurrent()

                    Dim power As Double = volt * amps

                    Console.WriteLine("Power output = " & Format(power / 1000000, "0.00") & " W")


                Case "U", "V"
                    Console.WriteLine("U (act) = " & BKcomm.mV_to_V(myBK.getPresentVoltage) & " V")
                    Thread.Sleep(100)
                    Console.WriteLine("U (set) = " & BKcomm.mV_to_V(myBK.getSetupVoltage) & " V")

                Case "I", "A"
                    Console.WriteLine("I (act) = " & BKcomm.mA_to_A(myBK.getPresentCurrent) & " A")
                    Thread.Sleep(100)
                    Console.WriteLine("I (set) = " & BKcomm.mA_to_A(myBK.getSetupCurrent) & " A")

                Case "HELP"
                    Console.WriteLine("Available list with commands:")
                    Console.WriteLine("EXIT         : quits the application")
                    Console.WriteLine("CONNECT      : connect to the BK instrument")
                    Console.WriteLine("DISCONNECT   : disconnect from the BK instrument")
                    Console.WriteLine("CLS          : clear the screen")
                    Console.WriteLine("REMOTE       : activate remote mode")
                    Console.WriteLine("LOCAL        : activate local mode")
                    Console.WriteLine("ON           : activate output")
                    Console.WriteLine("OFF          : deactivate output")
                    Console.WriteLine("??? V        : set output voltage")
                    Console.WriteLine("??? A        : set output current")
                    Console.WriteLine("??? mV       : set output voltage (mV)")
                    Console.WriteLine("??? mA       : set output voltage (mA)")

                Case Else
                    ' All other commands
                    Dim unit As New PhysicalUnit(newCmd)

                    ' Voltage entered?
                    If unit.isNumberOfUnit("V") Then
                        Dim voltage As Double
                        If unit.extractValue(voltage, "V") Then
                            If myBK.setOutputVoltage(voltage) Then
                                Console.WriteLine("Setting output to " & voltage & " mV")
                            Else
                                Console.WriteLine("Error: command could not be sent. Are you connected?")
                            End If
                        End If

                        ' Ampèrage entered?
                    ElseIf unit.isNumberOfUnit("A") Then
                        Dim amperage As Double
                        If unit.extractValue(amperage, "A") Then
                            If myBK.setOutputCurrent(amperage) Then
                                Console.WriteLine("Setting output to " & amperage & " mA")
                            Else
                                Console.WriteLine("Error: command could not be sent. Are you connected?")
                            End If
                        End If

                    Else
                        Console.WriteLine("Error: Unknown command")

                    End If
            End Select

            Console.WriteLine("")
        Loop

        ' Line added for debugging purposes. You can remove it if you want.
        Console.ReadLine()
    End Sub




    ' GetInput will read the entered data, convert to uppercase, and split the String into an Array.
    Private Function getInput(ByVal text As String) As String
        Console.Write(text)
        Dim commandText As String = Console.ReadLine().ToUpper().Trim()
        commandText = commandText.Replace(vbTab, " ")
        commandText = commandText.Replace(" ", "")
        getInput = commandText
    End Function
End Module